@@ -1,4 +1,4 @@
-package Net::FullAuto::ISets::Amazon::Chaining_is;
+package Net::FullAuto::ISets::Local::Chaining_is;
 
 ### OPEN SOURCE LICENSE - GNU AFFERO PUBLIC LICENSE Version 3.0 #######
 #
@@ -23,7 +23,7 @@ package Net::FullAuto::ISets::Amazon::Chaining_is;
 
 
 our $VERSION='0.01';
-our $DISPLAY='FullAuto© Proxy Chaining & RESTful Access';
+our $DISPLAY='FullAuto Proxy Chaining & RESTful Access';
 our $CONNECT='secure';
 our $defaultInstanceType='t2.small';
 
@@ -38,26 +38,72 @@ our @ISA = qw(Exporter);
 our @EXPORT = qw($select_chaining_setup);
 
 use Net::FullAuto::Cloud::fa_amazon;
-use Net::FullAuto::FA_Core qw[cmd_raw];
+use Net::FullAuto::FA_Core qw[cmd_raw $localhost];
+use File::HomeDir;
+my $home_dir=File::HomeDir->my_home.'/';
+my $username=getlogin || getpwuid($<);
+my $do;my $ad;my $prompt;
 
 my $configure_chaining=sub {
 
-   my $server_type=$_[0];
-   my $cnt=$_[1];
-   my $selection=$_[2]||'';
-   my $handle=$main::aws->{$server_type}->[$cnt]->[1];
+   my $selection=$_[0]||'';
    my ($stdout,$stderr)=('','');
-   ($stdout,$stderr)=$handle->cmd("sudo yum clean all");
-   ($stdout,$stderr)=$handle->cmd("sudo yum grouplist hidden");
-   ($stdout,$stderr)=$handle->cmd("sudo yum groups mark convert");
-   ($stdout,$stderr)=$handle->cmd(
-      "sudo yum -y groupinstall 'Development tools'",'__display__');
-   ($stdout,$stderr)=$handle->cmd(
-      'sudo yum -y install openssl-devel icu cyrus-sasl'.
-      ' libicu cyrus-sasl-devel libtool-ltdl-devel libxml2-devel'.
-      ' freetype-devel libpng-devel java-1.7.0-openjdk-devel'.
-      ' unixODBC unixODBC-devel libtool-ltdl libtool-ltdl-devel'.
-      ' ncurses-devel xmlto','__display__');
+   my $handle=$localhost;my $connect_error='';
+   $localhost->cwd('~');
+   my $sudo=($^O eq 'cygwin')?'':'sudo ';
+$do=1;
+if ($do==1) {
+   unless ($^O eq 'cygwin') {
+      ($stdout,$stderr)=$handle->cmd("sudo yum clean all");
+      ($stdout,$stderr)=$handle->cmd("sudo yum grouplist hidden");
+      ($stdout,$stderr)=$handle->cmd("sudo yum groups mark convert");
+      ($stdout,$stderr)=$handle->cmd(
+         "sudo yum -y groupinstall 'Development tools'",'__display__');
+      ($stdout,$stderr)=$handle->cmd(
+         'sudo yum -y install openssl-devel icu cyrus-sasl'.
+         ' libicu cyrus-sasl-devel libtool-ltdl-devel libxml2-devel'.
+         ' freetype-devel libpng-devel java-1.7.0-openjdk-devel'.
+         ' unixODBC unixODBC-devel libtool-ltdl libtool-ltdl-devel'.
+         ' ncurses-devel xmlto git-all','__display__');
+   } else {
+      my $cygcheck=`/bin/cygcheck -c` || die $!;
+      my $uname=`/bin/uname` || die $!;
+      my $uname_all=`/bin/uname -a` || die $!;
+      $uname_all.=$uname;
+      my %need_packages=();
+      if ($uname_all=~/x86_64/) {
+         foreach my $package ('libxml2','libxml2-devel','libtool',
+               'autoconf','autobuild','automake','pkg-config',
+               'libuuid-devel','wget','git') {
+            unless (-1<index $cygcheck, "$package ") {
+               $need_packages{$package}='';
+            }
+         }
+      } else {
+         foreach my $package ('libxml2','libxml2-devel','libtool',
+               'autoconf','autobuild','automake','pkg-config',
+               'libuuid-devel','wget','git') {
+            unless (-1<index $cygcheck, "$package ") {
+               $need_packages{$package}='';
+            }
+         }
+      }
+      my $packs='';
+      foreach my $pack (sort keys %need_packages) {
+         $packs.="$pack ";
+      }
+      if ($packs) {
+         print "\n\n   Fatal Error!: The following Cygwin",
+               "\n                 packages are missing from",
+               "\n                 your installation:",
+               "\n\n   $packs",
+               "\n\n   Please report any bugs and send any",
+               "\n   questions, thoughts or feedback to:",
+               "\n\n      Brian.Kelly\@FullAuto.com.",
+               "\n\n";
+         &Net::FullAuto::FA_Core::cleanup;
+      }
+   }
    ###############
    ## RABBITMQ
    ###############
@@ -98,7 +144,7 @@ my $configure_chaining=sub {
    #($stdout,$stderr)=$handle->cmd("rm -rvf $rmqtar",'__display__');
    #($stdout,$stderr)=$handle->cwd($rmqdir);
    #$handle->{_cmd_handle}->print('sudo su');
-   #my $prompt=substr($handle->{_cmd_handle}->prompt(),1,-1);
+   #$prompt=substr($handle->{_cmd_handle}->prompt(),1,-1);
    #while (1) {
    #   my $output.=Net::FullAuto::FA_Core::fetch($handle);
    #   last if $output=~/$prompt/;
@@ -143,57 +189,143 @@ my $configure_chaining=sub {
    #($stdout,$stderr)=$handle->cmd("rm -rvf master.zip",'__display__');
    #($stdout,$stderr)=$handle->cmd("sudo rabbitmq-server -detached",
    #   '__display__');
-   ($stdout,$stderr)=$handle->cmd(
-      "wget --random-wait --progress=dot ".
-      "http://download.fedoraproject.org".
-      "/pub/epel/6/x86_64/epel-release-6-8.noarch.rpm",
-      '__display__');
-   ($stdout,$stderr)=$handle->cmd("sudo rpm -ivh epel-release-6-8.noarch.rpm",
-      '__display__');
-   ($stdout,$stderr)=$handle->cmd("sudo rm -rvf epel-release-6-8.noarch.rpm",
-      '__display__');
-   ($stdout,$stderr)=$handle->cmd('sudo yum -y install uuid-devel '.
-      'pkgconfig libtool gcc-c++','__display__');
+   unless ($^O eq 'cygwin') {
+      ($stdout,$stderr)=$handle->cmd(
+         "wget --random-wait --progress=dot ".
+         "http://download.fedoraproject.org".
+         "/pub/epel/6/x86_64/epel-release-6-8.noarch.rpm",
+         '__display__');
+      ($stdout,$stderr)=$handle->cmd(
+         "sudo rpm -ivh epel-release-6-8.noarch.rpm",
+         '__display__');
+      ($stdout,$stderr)=$handle->cmd("sudo rm -rvf epel-release-6-8.noarch.rpm",
+         '__display__');
+      ($stdout,$stderr)=$handle->cmd('sudo yum -y install uuid-devel '.
+         'pkgconfig libtool gcc-c++','__display__');
+   }
    ($stdout,$stderr)=$handle->cmd(
       "wget --random-wait --progress=dot ".
       "https://github.com/jedisct1/libsodium/archive/master.zip",
       '__display__');
    ($stdout,$stderr)=$handle->cmd("unzip master.zip",'__display__');
-   ($stdout,$stderr)=$handle->cmd("sudo rm -rvf master.zip",'__display__');
+   ($stdout,$stderr)=$handle->cmd("${sudo}rm -rvf master.zip",'__display__');
    ($stdout,$stderr)=$handle->cwd("libsodium-master");
    ($stdout,$stderr)=$handle->cmd("./autogen.sh",'__display__');
    ($stdout,$stderr)=$handle->cmd("./configure",'__display__');
    ($stdout,$stderr)=$handle->cmd("make",'__display__');
-   ($stdout,$stderr)=$handle->cmd("sudo make install",'__display__');
+   ($stdout,$stderr)=$handle->cmd("${sudo}make install",'__display__');
    ($stdout,$stderr)=$handle->cwd("..");
-   ($stdout,$stderr)=$handle->cmd('echo /usr/local/lib > '.
-      'local.conf','__display__');
-   ($stdout,$stderr)=$handle->cmd('sudo chmod -v 644 local.conf',
-      '__display__');
-   ($stdout,$stderr)=$handle->cmd(
-      'sudo mv -v local.conf /etc/ld.so.conf.d','__display__');
-   ($stdout,$stderr)=$handle->cmd('sudo ldconfig');
+   unless ($^O eq 'cygwin') {
+      ($stdout,$stderr)=$handle->cmd('echo /usr/local/lib > '.
+         'local.conf','__display__');
+      ($stdout,$stderr)=$handle->cmd("${sudo}chmod -v 644 local.conf",
+         '__display__');
+      ($stdout,$stderr)=$handle->cmd(
+         "${sudo}mv -v local.conf /etc/ld.so.conf.d",'__display__');
+      ($stdout,$stderr)=$handle->cmd("${sudo}ldconfig");
+   }
+}
+$do=1;
+if ($do==1) {
+   my $zmq='zeromq-4.1.4.tar.gz';
    ($stdout,$stderr)=$handle->cmd(
       "wget --random-wait --progress=dot ".
-      "http://download.zeromq.org/zeromq-4.1.3.tar.gz",
+      "http://download.zeromq.org/$zmq",
       '__display__');
-   ($stdout,$stderr)=$handle->cmd('tar zxvf zeromq-4.1.3.tar.gz',
+   ($stdout,$stderr)=$handle->cmd("wget -qO- ".
+      "http://download.zeromq.org/SHA1SUMS");
+   my $zmqsha1=$stdout;
+   $zmqsha1=~s/^.*gz\s+(.*?)\s+$zmq.*$/$1/s;
+   ($stdout,$stderr)=$handle->cmd("sha1sum -c - <<<\"$zmqsha1 $zmq\"",
       '__display__');
-   ($stdout,$stderr)=$handle->cmd('rm -rvf zeromq-4.1.3.tar.gz',
-      '__display__');
-   ($stdout,$stderr)=$handle->cwd('zeromq-4.1.3');
+   unless ($stderr) {
+      print(qq{ + CHECKSUM Test for $zmq *PASSED* \n});
+   } else {
+      ($stdout,$stderr)=$handle->cmd("sudo rm -rvf $zmq",'__display__');
+      print "FATAL ERROR! : CHECKSUM Test for $zmq *FAILED* ";
+      &Net::FullAuto::FA_Core::cleanup;
+   }
+   ($stdout,$stderr)=$handle->cmd("tar zxvf $zmq",'__display__');
+   ($stdout,$stderr)=$handle->cmd("rm -rvf $zmq",'__display__');
+   $zmq=~s/\.tar\.gz$//;
+   ($stdout,$stderr)=$handle->cwd($zmq);
    cmd_raw($handle,'export PKG_CONFIG_PATH=/usr/local/lib/pkgconfig');
-   my $ad="Defaults    env_keep += \"PKG_CONFIG_PATH\"";
-   ($stdout,$stderr)=$handle->cmd(
-      "sudo sed -i \'/_XKB_CHARSET/a$ad\' /etc/sudoers");
    ($stdout,$stderr)=$handle->cmd('./configure','__display__');
+   if ($^O eq 'cygwin') {
+      my $ad="        -no-undefined \\%NL%".
+             "        -avoid-version \\";
+      ($stdout,$stderr)=$handle->cmd(
+         "sed -i \'/^libzmq_la_LDFLAGS = \\/a$ad\' ./Makefile");
+      ($stdout,$stderr)=$handle->cmd( # bash shell specific
+         "sed -i \'s/%NL%/\'\"`echo \\\\\\n`/g\" ".
+         "./Makefile");
+   } else {
+      my $ad="Defaults    env_keep += \"PKG_CONFIG_PATH\"";
+      ($stdout,$stderr)=$handle->cmd(
+         "sudo sed -i \'/_XKB_CHARSET/a$ad\' /etc/sudoers")
+   }
    ($stdout,$stderr)=$handle->cmd('make','__display__');
-   ($stdout,$stderr)=$handle->cmd('sudo make install','__display__');
+   ($stdout,$stderr)=$handle->cmd("${sudo}make install",'__display__');
    ($stdout,$stderr)=$handle->cwd("..");
-   ($stdout,$stderr)=$handle->cmd('sudo yum -y install cpan',
+}
+$do=0;
+if ($do==1) {
+   my $go=$1;my $gosha1=$2;
+   ($stdout,$stderr)=$handle->cmd("wget -qO- https://golang.org/dl");
+   if ($^O eq 'cygwin') {
+      $stdout=~
+         /^.*?href=["]([^"]+windows-amd64.zip)["].*?[<]tt[>](.*?)[<].*$/s;
+      $go=$1;$gosha1=$2;
+   } else {
+      $stdout=~
+         /^.*?href=["]([^"]+linux-amd64.tar.gz)["].*?[<]tt[>](.*?)[<].*$/s;
+      $go=$1;$gosha1=$2;
+   }
+   ($stdout,$stderr)=$handle->cmd(
+      "wget --random-wait --progress=dot ".$go,
       '__display__');
-   $handle->{_cmd_handle}->print('sudo cpan');
-   my $prompt=substr($handle->{_cmd_handle}->prompt(),1,-1);
+   $go=~s/^.*\/(.*)$/$1/;
+   ($stdout,$stderr)=$handle->cmd("sha1sum -c - <<<\"$gosha1 $go\"",
+      '__display__');
+   unless ($stderr) {
+      print(qq{ + CHECKSUM Test for $go *PASSED* \n});
+   } else {
+      ($stdout,$stderr)=$handle->cmd("sudo rm -rvf $go",'__display__');
+      print "FATAL ERROR! : CHECKSUM Test for $go *FAILED* ";
+      &Net::FullAuto::FA_Core::cleanup;
+   }
+   if ($^O eq 'cygwin') {
+      ($stdout,$stderr)=$handle->cmd("unzip $go",'__display__');
+   } else {
+      ($stdout,$stderr)=$handle->cmd("tar zxvf $go",'__display__');
+   }
+   ($stdout,$stderr)=$handle->cmd("rm -rvf $go",'__display__');
+}
+$do=0;
+if ($do==1) {
+   ($stdout,$stderr)=$handle->cmd('wget -qO- '.
+      'https://github.com/membrane/service-proxy/releases/latest');
+   $stdout=~s/^.*?href=["]([^"]+zip)["].*$/$1/s;
+   my $membrane_zip=$stdout;
+   ($stdout,$stderr)=$handle->cmd(
+      "wget --random-wait --progress=dot https://github.com".$membrane_zip,
+      '__display__');
+   $membrane_zip=~s/^.*\/(.*)$/$1/;
+   ($stdout,$stderr)=$handle->cmd("unzip $membrane_zip",'__display__');
+   ($stdout,$stderr)=$handle->cmd("rm -rvf $membrane_zip",'__display__');
+   #($stdout,$stderr)=$handle->cmd('git clone --depth=1 '.
+   #   'https://github.com/membrane/service-proxy.git','__display__');
+exit;
+}
+   if ($^O eq 'cygwin') {
+      ($stdout,$stderr)=$handle->cwd("~");
+      $handle->{_cmd_handle}->print('cpan');
+   } else {
+      ($stdout,$stderr)=$handle->cmd('sudo yum -y install cpan',
+         '__display__');
+      $handle->{_cmd_handle}->print('sudo cpan');
+   } 
+   $prompt=substr($handle->{_cmd_handle}->prompt(),1,-1);
    while (1) {
       my $output.=Net::FullAuto::FA_Core::fetch($handle);
       last if $output=~/$prompt/;
@@ -228,9 +360,47 @@ my $configure_chaining=sub {
       }
    }
    ($stdout,$stderr)=$handle->cmd("export PERL_MM_USE_DEFAULT=1");
+   if ($^O eq 'cygwin') {
+      my $show=<<END;
+########################################
+
+   INSTALLING Starman
+
+########################################
+END
+      print $show;
+      cmd_raw($handle,
+         'perl -MCPAN -e \'CPAN::Shell->notest('.
+         '"install","Starman")\'',
+         '__display__');
+      $show=<<END;
+########################################
+
+   INSTALLING HTTP::Server::Simple
+
+########################################
+END
+      print $show;
+      cmd_raw($handle,
+         'perl -MCPAN -e \'CPAN::Shell->notest('.
+         '"install","HTTP::Server::Simple")\'',
+         '__display__');
+   }
    my $show=<<END;
 ########################################
 
+   INSTALLING Perl::Critic
+
+########################################
+END
+   print $show;
+   cmd_raw($handle,
+      'perl -MCPAN -e \'CPAN::Shell->notest('.
+      '"install","Perl::Critic")\'',
+      '__display__');
+   $show=<<END;
+########################################
+
    INSTALLING ExtUtils::Embed
 
 ########################################
@@ -344,7 +514,7 @@ END
                \__/_     ___ __ _| |_ __ _| |_   _ ___| |_
                /    \   / __/ _` | __/ _` | | | | / __| __|  Perl MVC
             _- |    |  | (_| (_| | || (_| | | |_| \__ \ |    framework
-       _ _-'   \____/   \___\__,_|\__\__,_|_|\__, |___/\__|©
+       _ _-'   \____/   \___\__,_|\__\__,_|_|\__, |___/\__|c
      ((_)       ---\                         |___/
                     \
                      \\_          Web Framework
@@ -367,11 +537,16 @@ END
          print $install_chaining;
          sleep 10;
       }
-      cmd_raw($handle,
-         "sudo LD_LIBRARY_PATH=/usr/local/lib ".
-         "PKG_CONFIG_PATH=/usr/local/lib/pkgconfig ".
-         "LD_PRELOAD=/usr/local/lib/libzmq.so.5 cpan -f -i $module",
-         '__display__');
+      unless ($^O eq 'cygwin') {
+         cmd_raw($handle,
+            "${sudo}LD_LIBRARY_PATH=/usr/local/lib ".
+            "PKG_CONFIG_PATH=/usr/local/lib/pkgconfig ".
+            "LD_PRELOAD=/usr/local/lib/libzmq.so.5 cpan -f -i $module",
+            '__display__');
+      } else {
+         cmd_raw($handle,"cpan -i $module",
+            '__display__');
+      }
    }
    $show=<<END;
 ########################################
@@ -386,6 +561,7 @@ END
 #      '"install","Regexp::Assemble")\'',
 #      '__display__');
    $show=<<END;
+
 ########################################
 
    INSTALLING Catalyst::Devel
@@ -393,7 +569,7 @@ END
 ########################################
 END
    print $show;
-   $handle->{_cmd_handle}->print('sudo cpan Catalyst::Devel');
+   $handle->{_cmd_handle}->print("${sudo}cpan Catalyst::Devel");
    $prompt=substr($handle->{_cmd_handle}->prompt(),1,-1);
    while (1) {
       my $output.=Net::FullAuto::FA_Core::fetch($handle);
@@ -411,26 +587,29 @@ END
       }
    }
    $show=<<END;
+
 ########################################
 
    INSTALLING Catalyst::Controller::HTML::FormFu
 
 ########################################
 END
-   cmd_raw($handle,'sudo cpan Catalyst::Controller::HTML::FormFu',
+   cmd_raw($handle,"${sudo}cpan Catalyst::Controller::HTML::FormFu",
       '__display__');
    print "\n";
    $show=<<END;
+
 ########################################
 
    INSTALLING CatalystX::OAuth2
 
 ########################################
 END
-   cmd_raw($handle,'sudo cpan CatalystX::OAuth2',
+   cmd_raw($handle,"${sudo}cpan CatalystX::OAuth2",
       '__display__');
    print "\n";
    $show=<<END;
+
 ########################################
 
    INSTALLING Task::Catalyst::Tutorial
@@ -439,17 +618,19 @@ END
 END
    print $show;
    sleep 1;
-   cmd_raw($handle,'sudo cpan Task::Catalyst::Tutorial','__display__');
+   cmd_raw($handle,"${sudo}cpan Task::Catalyst::Tutorial",'__display__');
    $show=<<END;
+
 ########################################
 
    INSTALLING DBIx::Class::Schema::Loader
 
 ########################################
 END
-   cmd_raw($handle,'sudo cpan DBIx::Class::Schema::Loader',
+   cmd_raw($handle,"${sudo}cpan DBIx::Class::Schema::Loader",
       '__display__');
 #   $show=<<END;
+#
 ########################################
 #
 #   INSTALLING Net::RabbitFoot
@@ -457,7 +638,7 @@ END
 ########################################
 #END
 #   print $show;
-#   $handle->{_cmd_handle}->print('sudo cpan Net::RabbitFoot');
+#   $handle->{_cmd_handle}->print("${sudo}cpan Net::RabbitFoot");
 #   $prompt=substr($handle->{_cmd_handle}->prompt(),1,-1);
 #   while (1) {
 #      my $output.=Net::FullAuto::FA_Core::fetch($handle);
@@ -470,6 +651,7 @@ END
 #      }
 #   }
    $show=<<END;
+
 ########################################
 
    INSTALLING YAML::Syck
@@ -478,8 +660,9 @@ END
 END
    print $show;
    sleep 1;
-   cmd_raw($handle,'sudo cpan YAML::Syck','__display__');
+   cmd_raw($handle,"${sudo}cpan YAML::Syck",'__display__');
    $show=<<END;
+
 ########################################
 
    INSTALLING Catalyst::Controller::REST
@@ -488,8 +671,9 @@ END
 END
    print $show;
    sleep 1;
-   cmd_raw($handle,'sudo cpan Catalyst::Controller::REST','__display__');
+   cmd_raw($handle,"${sudo}cpan Catalyst::Controller::REST",'__display__');
    $show=<<END;
+
 ########################################
 
    INSTALLING Catalyst::Model::Adaptor
@@ -498,8 +682,9 @@ END
 END
    print $show;
    sleep 1;
-   cmd_raw($handle,'sudo cpan Catalyst::Model::Adaptor','__display__');
+   cmd_raw($handle,"${sudo}cpan Catalyst::Model::Adaptor",'__display__');
    $show=<<END;
+
 ########################################
 
    INSTALLING Catalyst::View::JSON
@@ -508,8 +693,9 @@ END
 END
    print $show;
    sleep 1;
-   cmd_raw($handle,'sudo cpan Catalyst::View::JSON','__display__');
+   cmd_raw($handle,"${sudo}cpan Catalyst::View::JSON",'__display__');
    $show=<<END;
+
 ########################################
 
    INSTALLING Catalyst::View::TT::Alloy
@@ -518,8 +704,9 @@ END
 END
    print $show;
    sleep 1;
-   cmd_raw($handle,'sudo cpan Catalyst::View::TT::Alloy','__display__');
+   cmd_raw($handle,"${sudo}cpan Catalyst::View::TT::Alloy",'__display__');
    $show=<<END;
+
 ########################################
 
    INSTALLING Catalyst::Plugin::Unicode
@@ -528,8 +715,9 @@ END
 END
    print $show;
    sleep 1;
-   cmd_raw($handle,'sudo cpan Catalyst::Plugin::Unicode','__display__');
+   cmd_raw($handle,"${sudo}cpan Catalyst::Plugin::Unicode",'__display__');
    $show=<<END;
+
 ########################################
 
    INSTALLING Finance::Quote
@@ -537,7 +725,7 @@ END
 ########################################
 END
    print $show;
-   $handle->{_cmd_handle}->print('sudo cpan Finance::Quote');
+   $handle->{_cmd_handle}->print("${sudo}cpan Finance::Quote");
    $prompt=substr($handle->{_cmd_handle}->prompt(),1,-1);
    while (1) {
       my $output.=Net::FullAuto::FA_Core::fetch($handle);
@@ -634,7 +822,7 @@ END
    ($stdout,$stderr)=$handle->cmd("./script/adventrest_create.pl ".
       "controller OAuth2::Provider",'__display__');
    my $pro_path="./lib/AdventREST/Controller/OAuth2/Provider.pm";
-   ($stdout,$stderr)=$handle->cmd('sudo sed -i '.
+   ($stdout,$stderr)=$handle->cmd("${sudo}sed -i ".
       "\"s/Catalyst::Controller/Catalyst::Controller::ActionRole/\" ".
       $pro_path);
    $ad="%NL%".
@@ -666,10 +854,10 @@ END
        "sub refresh : Chained(%SQ%/%SQ%) Args(0) ".
        "Does(%SQ%OAuth2::AuthToken::ViaRefreshToken%SQ%) {}%NL%";
    ($stdout,$stderr)=$handle->cmd(
-      "sudo sed -i '/ActionRole/a$ad' $pro_path");
+      "${sudo}sed -i '/ActionRole/a$ad' $pro_path");
    ($stdout,$stderr)=$handle->cmd( # bash shell specific
-      "sudo sed -i \'s/%NL%/\'\"`echo \\\\\\n`/g\" $pro_path");
-   ($stdout,$stderr)=$handle->cmd("sudo sed -i \"s/%SQ%/\'/g\" $pro_path");
+      "${sudo}sed -i \'s/%NL%/\'\"`echo \\\\\\n`/g\" $pro_path");
+   ($stdout,$stderr)=$handle->cmd("${sudo}sed -i \"s/%SQ%/\'/g\" $pro_path");
    $content=<<'END';
 name: AdventREST
 Model::DB:
@@ -707,13 +895,13 @@ END
       "https://github.com/DataTables/DataTables/archive/master.zip",
       '__display__');
    ($stdout,$stderr)=$handle->cmd('unzip master.zip','__display__');
-   ($stdout,$stderr)=$handle->cmd('sudo rm -rvf master.zip','__display__');
+   ($stdout,$stderr)=$handle->cmd("${sudo}rm -rvf master.zip",'__display__');
    ($stdout,$stderr)=$handle->cwd('DataTables-master');
    ($stdout,$stderr)=$handle->cmd('cp -Rv media ..','__display__');
    ($stdout,$stderr)=$handle->cwd('examples');
    ($stdout,$stderr)=$handle->cmd('cp -Rv resources ../..','__display__');
    ($stdout,$stderr)=$handle->cwd('../../..');
-   $content=<<'END';
+   $content=<<END;
 package AdventREST::Controller::User;
  
 use strict;
@@ -728,10 +916,10 @@ use YAML;
 use Carp::Assert;
 
 use constant NBR_WORKERS    => 2;
-use constant READY          => \\x22\\\\001\\x22;
+use constant READY          => \\x22\\\\\\\\001\\x22;
 
 use constant FRONTEND_URL   =>
-       'ipc:///home/ec2-user/AdventREST/frontend.ipc';
+       \\x22ipc://${home_dir}AdventREST/frontend.ipc\\x22;
 
 BEGIN { extends 'Catalyst::Controller::REST' }
 
@@ -756,7 +944,7 @@ __PACKAGE__->config(
 sub user_list : Path('/user') :Args(0) : ActionClass('REST') { }
 
 sub user_list_GET {
-    my ( \\x24self, \\x24c ) = @_;
+    my ( \\x24self, \\x24c ) = \@_;
     my \\x24draw     = \\x24c->req->params->{draw} || 0;
     my \\x24start    = \\x24c->req->params->{start} || 0;
     my \\x24per_page = \\x24c->req->params->{length} || 10;
@@ -779,21 +967,21 @@ sub user_list_GET {
 
     assert(\\x24rv == 0,'connecting client ...');
 
-    print \\x22\\x24id sending Hello\\\\n\\x22;
+    print \\x22\\x24id sending Hello\\\\\\\\n\\x22;
     \\x24rv         = zmq_msg_send('Hello',\\x24socket);
 
     my \\x24reply = zmq_recvmsg(\\x24socket);
 
     assert(\\x24reply);
 
-    print \\x22\\x24id got a reply -> \\x22,zmq_msg_data(\\x24reply),\\x22\\n\\x22;
+    print \\x22\\x24id got a reply -> \\x22,zmq_msg_data(\\x24reply),\\x22\\\\\\\\n\\x22;
 
     # We'll use an array now:
-    my @user_list;
+    my \@user_list;
     my \\x24rs = \\x24c->model('DB::User')
         ->search(undef, { rows => \\x24per_page })->page( \\x24page );
     while ( my \\x24user_row = \\x24rs->next ) {
-        push @user_list, {
+        push \@user_list, {
             \\x24user_row->get_columns,
             uri => \\x24c->uri_for( '/user/' . \\x24user_row->user_id )->as_string
         };
@@ -803,18 +991,18 @@ sub user_list_GET {
         draw => \\x24draw,
         recordsTotal => \\x24rs->pager->total_entries,
         recordsFiltered => \\x24rs->pager->total_entries,
-        data => [ @user_list ]
+        data => [ \@user_list ]
     });
 };
 
 sub single_user : Path('/user') : Args(1) : ActionClass('REST') {
-    my ( \\x24self, \\x24c, \\x24user_id ) = @_;
+    my ( \\x24self, \\x24c, \\x24user_id ) = \@_;
  
     \\x24c->stash->{'user'} = \\x24c->model('DB::User')->find(\\x24user_id);
 }
 
 sub single_user_POST {
-    my ( \\x24self, \\x24c, \\x24user_id ) = @_;
+    my ( \\x24self, \\x24c, \\x24user_id ) = \@_;
  
     my \\x24new_user_data = \\x24c->req->data;
     if ( \\x21defined(\\x24new_user_data) ) {
@@ -863,7 +1051,7 @@ sub single_user_POST {
 *single_user_PUT = *single_user_POST;
 
 sub single_user_GET {
-    my ( \\x24self, \\x24c, \\x24user_id ) = @_;
+    my ( \\x24self, \\x24c, \\x24user_id ) = \@_;
  
     my \\x24user = \\x24c->stash->{'user'};
     if ( defined(\\x24user) ) {
@@ -883,7 +1071,7 @@ sub single_user_GET {
 }
 
 sub single_user_DELETE {
-    my ( \\x24self, \\x24c, \\x24user_id ) = @_;
+    my ( \\x24self, \\x24c, \\x24user_id ) = \@_;
  
     my \\x24user = \\x24c->stash->{'user'};
     if ( defined(\\x24user) ) {
@@ -911,12 +1099,12 @@ END
    $ad='sub index : Private {%NL%   my ( $self, $c ) = @_;%NL%'.
           '   $c->forward( $c->view(%SQ%TT%SQ%) );%NL%}%NL%%NL%';
    ($stdout,$stderr)=$handle->cmd(
-      "sudo sed -i '/sub index :Path/i$ad' ./Root.pm");
+      "${sudo}sed -i '/sub index :Path/i$ad' ./Root.pm");
    ($stdout,$stderr)=$handle->cmd( # bash shell specific
-      "sudo sed -i \'s/%NL%/\'\"`echo \\\\\\n`/g\" ./Root.pm");
-   ($stdout,$stderr)=$handle->cmd("sudo sed -i \"s/%SQ%/\'/g\" ./Root.pm");
+      "${sudo}sed -i \'s/%NL%/\'\"`echo \\\\\\n`/g\" ./Root.pm");
+   ($stdout,$stderr)=$handle->cmd("${sudo}sed -i \"s/%SQ%/\'/g\" ./Root.pm");
    ($stdout,$stderr)=$handle->cmd(
-      "sudo sed -i '/index :Path :/,+6d' ./Root.pm");
+      "${sudo}sed -i '/index :Path :/,+6d' ./Root.pm");
    ($stdout,$stderr)=$handle->cmd("chmod -v 644 User.pm",'__display__');
    ($stdout,$stderr)=$handle->cwd('../../../root/static');
    ($stdout,$stderr)=$handle->cmd(
@@ -1074,10 +1262,10 @@ END
    $ad='__PACKAGE__->config( class => "TheSchwartz" );%NL%%NL%'. # %NL% is newline
       'sub mangle_arguments { %{ $_[1]->{args} } }%NL%';
    ($stdout,$stderr)=$handle->cmd(
-      "sudo sed -i \'/Catalyst::Model/a$ad\' ".
+      "${sudo}sed -i \'/Catalyst::Model/a$ad\' ".
       "./lib/AdventREST/Model/TheSchwartz.pm");
    ($stdout,$stderr)=$handle->cmd( # bash shell specific
-      "sudo sed -i \'s/%NL%/\'\"`echo \\\\\\n`/g\" ".
+      "${sudo}sed -i \'s/%NL%/\'\"`echo \\\\\\n`/g\" ".
       "./lib/AdventREST/Model/TheSchwartz.pm");
    $content=<<END;
 Model::TheSchwartz:
@@ -1096,9 +1284,9 @@ END
        "use constant READY          => \"\\\\001\";%NL%".
        "%NL%".
        "use constant BACKEND_URL    =>%NL%".
-       "       %SQ%ipc:///home/ec2-user/AdventREST/backend.ipc%SQ%;%NL%".
+       "       %SQ%ipc://${home_dir}AdventREST/backend.ipc%SQ%;%NL%".
        "use constant FRONTEND_URL    =>%NL%".
-       "       %SQ%ipc:///home/ec2-user/AdventREST/frontend.ipc%SQ%;%NL%".
+       "       %SQ%ipc://${home_dir}AdventREST/frontend.ipc%SQ%;%NL%".
        "%NL%".
        "use Parallel::Forker;%NL%".
        "%NL%".
@@ -1107,15 +1295,15 @@ END
        "   my \$server={%NL%".
        "%NL%".
        "      Label => %SQ%server%SQ%,%NL%".
-       "      LoginID => %SQ%ec2-user%SQ%,%NL%".
-       "      IdentityFile => %SQ%/home/ec2-user/fullauto.pem%SQ%,%NL%".
+       "      LoginID => %SQ%${username}%SQ%,%NL%".
+       "      IdentityFile => %SQ%${home_dir}fullauto.pem%SQ%,%NL%".
        "      HostName => %SQ%localhost%SQ%,%NL%".
        "%NL%".
        "   };%NL%".
        "   # A bit of custom config riding/hacking to use the application".
        "%SQ%s%NL%".
        "   # config for the DB.%NL%".
-       "   my \$config_file = %SQ%/home/ec2-user/AdventREST/adventrest.yml".
+       "   my \$config_file = %SQ%${home_dir}AdventREST/adventrest.yml".
        "%SQ%;%NL%".
        "   my \$config_data = YAML::LoadFile( \$config_file );%NL%".
        "   #my \$args = \$config_data->{\"Model::ZeroMQ\"}->{args};%NL%".
@@ -1136,7 +1324,7 @@ END
        "%NL%".
        "   my (\$fullauto,\$stdout,\$stderr,\$exitcode,".
        "\$connect_error)=(%SQ%%SQ%,%SQ%%SQ%,%SQ%%SQ%,%SQ%%SQ%,%SQ%%SQ%);%NL%".
-       "   (\$fullauto,\$connect_error)=connect_ssh(\$server);%NL%".
+       "   (\$fullauto,\$connect_error)=connect_shell(\$server);%NL%".
        "%NL%".
        "   use Time::HiRes;%NL%".
        "   # http://www.unitconversion.org/unit_converter/time-ex.html%NL%".
@@ -1293,12 +1481,12 @@ END
        "%NL%".
        "}%NL%";
    ($stdout,$stderr)=$handle->cmd(
-      "sudo sed -i \'/use Catalyst::ScriptRunner/i$ad\' ".
+      "${sudo}sed -i \'/use Catalyst::ScriptRunner/i$ad\' ".
       "./script/adventrest_server.pl");
    ($stdout,$stderr)=$handle->cmd( # bash shell specific
-      "sudo sed -i \'s/%NL%/\'\"`echo \\\\\\n`/g\" ".
+      "${sudo}sed -i \'s/%NL%/\'\"`echo \\\\\\n`/g\" ".
       "./script/adventrest_server.pl");
-   ($stdout,$stderr)=$handle->cmd("sudo sed -i \"s/%SQ%/\'/g\" ".
+   ($stdout,$stderr)=$handle->cmd("${sudo}sed -i \"s/%SQ%/\'/g\" ".
       "./script/adventrest_server.pl");
    ($stdout,$stderr)=$handle->cwd("..");
    ($stdout,$stderr)=$handle->put("fullauto.pem");
@@ -1323,12 +1511,14 @@ END
    $ad='encoding: utf8';
    ($stdout,$stderr)=$handle->cmd(
       "sed -i \'/default_view/a $ad\' myapp.yml");
+print "GOING TO START MYAPP SERVER\n";
    $handle->{_cmd_handle}->print(
       'script/myapp_server.pl --background');
    ($stdout,$stderr)=$handle->cwd("../AdventREST");
-   ($stdout,$stderr)=$handle->cmd('sudo ldconfig');
+   ($stdout,$stderr)=$handle->cmd("${sudo}ldconfig");
+print "GOING TO START WEB SERVER\n";
    $handle->{_cmd_handle}->print(
-      'sudo script/adventrest_server.pl -p 3001 &');
+      "${sudo}script/adventrest_server.pl -p 3001 &");
    $prompt=substr($handle->{_cmd_handle}->prompt(),1,-1);
    while (1) {
       my $output.=Net::FullAuto::FA_Core::fetch($handle);
@@ -1343,14 +1533,9 @@ END
          last;
       }
    }
-   my $master=$main::aws->{$server_type}->[0]->[0]->{InstanceId};
-   my $c="aws ec2 describe-instances --instance-ids $master 2>&1";
-   my ($hash,$output,$error)=('','','');
-   ($hash,$output,$error)=run_aws_cmd($c);
-   my $mdns=$hash->{Reservations}->[0]->{Instances}->[0]->{PublicDnsName};
    sleep 15;
    print "\n   ACCESS CATALYST REST EXAMPLE AT:\n\n",
-         " http://$mdns:3001\n";
+         " http://localhost:3001\n";
    my $thanks=<<'END';
 
      ______                  _    ,
@@ -1363,10 +1548,10 @@ END
           | \ | | ___| |_      |  ___|   _| | |  / \  _   _| |_  |
           |  \| |/ _ \ __| o o | |_ | | | | | | / _ \| | | | __/ | \
           | |\  |  __/ |_  o o |  _|| |_| | | |/ ___ \ |_| | ||     |
-          |_| \_|\___|\__|     |_|   \__,_|_|_/_/   \_\__,_|\__\___/ ©
+          |_| \_|\___|\__|     |_|   \__,_|_|_/_/   \_\__,_|\__\___/ (C)
 
 
-   Copyright © 2000-2016  Brian M. Kelly  Brian.Kelly@FullAuto.com
+   Copyright (C) 2000-2016  Brian M. Kelly  Brian.Kelly@FullAuto.com
 
 END
    eval {
@@ -1384,69 +1569,9 @@ END
 
 my $standup_chaining=sub {
 
-   my $type="]T[{select_type}";
-   $type=~s/^"//;
-   $type=~s/"$//;
-   $type=substr($type,0,(index $type,' ->')-3);
    my $catalyst="]T[{select_chaining_setup}";
-   my $i=$main::aws->{fullauto}->{ImageId}||'';
-   my $s=$main::aws->{fullauto}->
-         {NetworkInterfaces}->[0]->{SubnetId}||'';
-   my $g=$main::aws->{fullauto}->
-         {SecurityGroups}->[0]->{GroupId}||'';
-   my $n=$main::aws->{fullauto}->
-         {SecurityGroups}->[0]->{GroupName}||'';
-   my $c='aws ec2 describe-security-groups '.
-         "--group-names $n";
-   my ($hash,$output,$error)=('','','');
-   ($hash,$output,$error)=run_aws_cmd($c);
-   Net::FullAuto::FA_Core::handle_error($error) if $error;
-   my $cidr=$hash->{SecurityGroups}->[0]->{IpPermissions}
-            ->[0]->{IpRanges}->[0]->{CidrIp};
-   $c='aws ec2 create-security-group --group-name '.
-      'CatalystSecurityGroup --description '.
-      '"CatalystFramework.org Security Group" 2>&1';
-   ($hash,$output,$error)=run_aws_cmd($c);
-   Net::FullAuto::FA_Core::handle_error($error) if $error
-      && $error!~/already exists/;
-   $c='aws ec2 authorize-security-group-ingress '.
-      '--group-name CatalystSecurityGroup --protocol '.
-      'tcp --port 22 --cidr '.$cidr." 2>&1";
-   ($hash,$output,$error)=run_aws_cmd($c);
-   Net::FullAuto::FA_Core::handle_error($error) if $error
-      && $error!~/already exists/;
-   $c='aws ec2 authorize-security-group-ingress '.
-      '--group-name CatalystSecurityGroup --protocol '.
-      'tcp --port 3000 --cidr '.$cidr." 2>&1";
-   ($hash,$output,$error)=run_aws_cmd($c);
-   Net::FullAuto::FA_Core::handle_error($error) if $error
-      && $error!~/already exists/;
-   $c='aws ec2 authorize-security-group-ingress '.
-      '--group-name CatalystSecurityGroup --protocol '.
-      'tcp --port 3001 --cidr '.$cidr." 2>&1";
-   ($hash,$output,$error)=run_aws_cmd($c);
-   Net::FullAuto::FA_Core::handle_error($error) if $error
-      && $error!~/already exists/;
    my $cnt=0;
-   my $pemfile=$pem_file;
-   $pemfile=~s/\.pem\s*$//s;
-   if (exists $main::aws->{'CatalystFramework.org'}) {
-      my $g=get_aws_security_id('CatalystSecurityGroup');
-      my $c="aws ec2 run-instances --image-id $i --count 1 ".
-         "--instance-type $type --key-name $pemfile ".
-         "--security-group-ids $g --subnet-id $s";
-      if ($#{$main::aws->{'CatalystFramework.org'}}==0) {
-         launch_server('CatalystFramework.org',$cnt,$catalyst,'',$c,
-         $configure_chaining);
-      } else {
-         my $num=$#{$main::aws->{'CatalystFramework.org'}}-1;
-         foreach my $num (0..$num) {
-            launch_server('CatalystFramework',$cnt++,$catalyst,'',$c,
-            $configure_chaining);
-         }
-      }
-   }
-
+   $configure_chaining->($catalyst);
    return '{choose_demo_setup}<';
 
 };
@@ -1549,11 +1674,11 @@ our $select_chaining_setup=sub {
                      _    ___     _ _   _       _                     
                    ((_)  | __|  _| | | /_\ _  _| |_  |                   
                     /    | _| || | | |/ _ \ || |  _/ | \   Automates     _
-                   /     |_| \_,_|_|_/_/ \_\_,_|\__\___/©  Everything  _| |_ 
+                   /     |_| \_,_|_|_/_/ \_\_,_|\__\___/c  Everything  _| |_ 
                \__/_     ___ __ _| |_ __ _| |_   _ ___| |_            |_   _|
                /    \   / __/ _` | __/ _` | | | | / __| __|  Perl MVC   |_|
             _- |    |  | (_| (_| | || (_| | | |_| \__ \ |    framework
-       _ _-'   \____/   \___\__,_|\__\__,_|_|\__, |___/\__|©
+       _ _-'   \____/   \___\__,_|\__\__,_|_|\__, |___/\__|c
      ((_)       ---\                         |___/
                     \
                      \\_   Web Framework & SSH Proxy Chaining via RESTful
@@ -1573,7 +1698,7 @@ END
 
          Text => ']C[',
          Convey => \@options,
-         Result => $chaining_setup_summary,
+         Result => $standup_chaining,
 
       },
       Scroll => 1,
